@@ -40,12 +40,8 @@ class Player:
     def __init__(self, name):
         self.name = name
 
-    # returns None for pass
-    def make_bet(self, game, player: TurnPosition):
-        return None
-
     # returns None for pass, otherwise returns a SpecificMove
-    def make_move(self, game, player: TurnPosition):
+    def make_move(self, game):
         pass
 
 class LearningPlayer_v1(Player):
@@ -59,11 +55,15 @@ class LearningPlayer_v1(Player):
     # appends the length of each player's hand
     HAND_FEATURES = len(Card) + 3
 
-    def __init__(self, name, epsilon=0.1, learning_rate=0.2, discount_factor=0.8, net_dir=None):
+    def __init__(self, name, net_dir=None, epsilon=0.1, learning_rate=0.2, discount_factor=0.8,
+                  use_montecarlo_random=True, random_mc_num_explorations=30, mc_best_move_depth=1):
         super().__init__(name)
 
         self.epsilon = epsilon
         self.empty_nets = False
+        self.use_montecarlo_random = use_montecarlo_random
+        self.random_mc_num_explorations = random_mc_num_explorations
+        self.mc_best_move_depth = mc_best_move_depth
         if net_dir is None:
             self.empty_nets = True
         else:
@@ -81,6 +81,8 @@ class LearningPlayer_v1(Player):
         self.record_move_vectors = []
         self._record_future_q = []
         self.record_hand_vectors = []
+        # for debugging
+        self._record_state_q = []
 
     def load_nnets(self, net_dir):
         self.history_net = keras.models.load_model(net_dir + "/history.h5")
@@ -219,9 +221,20 @@ class LearningPlayer_v1(Player):
 
         return self.position_net.predict([history_matrix, move_options_matrix, hand_matrix], batch_size=1024).reshape(move_options_matrix.shape[0])
 
-    def decide_best_move(self, game, player: TurnPosition, debug=False):
-        assert len(game.get_hand(player)) > 0
-        legal_moves = game.get_legal_moves(player)
+    def make_bet_decision(self, game, legal_moves, predictions):
+        bet_indices = []
+        # issue with recursive import, so not using constant
+        flip_predictions = np.copy(predictions)
+        for i in range(3):
+            bet_index = legal_moves.index([move for move in legal_moves if move.get_amount() == i][0])
+            if i <= game.get_bet_amount():
+                flip_predictions[bet_index] = - flip_predictions[bet_index]
+
+        return np.argmax(flip_predictions)
+
+    def decide_best_move(self, game, debug=False):
+        assert len(game.get_hand(game.get_current_position())) > 0
+        legal_moves = game.get_legal_moves()
 
         history_features = self.derive_features(game)
 
@@ -230,7 +243,7 @@ class LearningPlayer_v1(Player):
         history_matrix = np.tile(history_vector, (len(legal_moves), 1))
 
         # make the hand vector and copy it
-        hand_vector = self.get_hand_vector(game, player)
+        hand_vector = self.get_hand_vector(game, game.get_current_position())
         hand_matrix = np.tile(hand_vector, (len(legal_moves), 1))
 
         # create features for each of the possible moves from this position
@@ -239,6 +252,9 @@ class LearningPlayer_v1(Player):
 
         predictions = self.get_position_predictions(history_matrix, move_options_matrix, hand_matrix)
 
+        # for debugging
+        raw_predictions = np.copy(predictions)
+
         # if the move ends the game, then force-score the position
         has_game_ending = False
         for i, move in enumerate(legal_moves):
@@ -249,21 +265,22 @@ class LearningPlayer_v1(Player):
                 has_game_ending = True
 
         best_move_index = 0
-        if player == game.get_landlord_position():
+        if game.get_current_position() == game.get_landlord_position():
             best_move_index = np.argmax(predictions)
         elif game.is_betting_complete():
             best_move_index = np.argmin(predictions)
         else:
             # go landlord if it's worth it, otherwise peasant
-            best_move_index = np.argmax(np.abs(predictions))
+            best_move_index = self.make_bet_decision(game, legal_moves, predictions)
 
         if debug:
-            print('Is Landlord', player == game.get_landlord_position())
-            print('Player', player)
-            print('Name', self.get_name())
-            print('Hand', game.get_hand(player))
-            for move, pred in sorted(list(zip(legal_moves, predictions)), key=lambda x : x[1]):
-                print(pred, move)
+            print('(' +  game.get_position_role_name(game.get_current_position()) + ') Player', self.get_name())
+            print('Hand', game.get_hand(game.get_current_position()))
+            for move, raw_pred, score in sorted(list(zip(legal_moves, raw_predictions, predictions)), key=lambda x : x[1]):
+                if raw_pred != score:
+                    print(raw_pred, score, move)
+                else:
+                    print(raw_pred, move)
             print('Made Move', legal_moves[best_move_index])
 
         if random.random() < self.epsilon and not has_game_ending:
@@ -272,50 +289,71 @@ class LearningPlayer_v1(Player):
                 print('Randomed', legal_moves[best_move_index])
 
         best_move = legal_moves[best_move_index]
-        best_move_q = predictions[best_move_index]
+        # return the predicted Q here, not the clamped Q
+        best_move_q = raw_predictions[best_move_index]
 
         return best_move, best_move_q
 
-    def make_bet(self, game, player: TurnPosition):
-        return self.make_move(game, player)
+    def make_move(self, game, debug=False):
+        best_move, best_move_q = self.decide_best_move(game, debug=debug)
 
-    def make_move(self, game, player: TurnPosition):
-        best_move, best_move_q = self.decide_best_move(game, player, debug=True)
-
-        self.record_move(game, best_move, best_move_q, player)
+        self.record_move(game, best_move, best_move_q, game.get_current_position())
 
         return best_move
 
+
+
+    # returns a future q based on random search
+    def monte_carlo_best_search(self, game, depth=6):
+        copy_game = copy(game)
+        best_next_move_q = 0
+        for i in range(depth):
+            best_next_move, best_next_move_q = self.decide_best_move(copy_game)
+            if copy_game.move_ends_game(best_next_move):
+                return best_next_move_q
+            else:
+                copy_game.play_move(best_next_move)
+
+        return best_next_move_q
+
+    def monte_carlo_random_search(self, game, num_explorations):
+        reward_values = []
+        for i in range(num_explorations):
+            copy_game = copy(game)
+            while not copy_game.is_round_over():
+                move = random.choice(copy_game.get_legal_moves())
+                copy_game.play_move(move)
+            reward_values.append(copy_game.get_r())
+        return np.mean(reward_values)
+
     def record_move(self, game, best_move, best_move_q, player: TurnPosition):
-        previous_history_matrix, move_vector = self.derive_record_features(game)
+        #previous_history_matrix, move_vector = self.derive_record_features(game)
+        history_matrix = self.derive_features(game)
+        move_vector = self.compute_move_vector(player, game.get_landlord_position(), best_move)
         hand_vector = self.get_hand_vector(game, player)
 
         copy_game = copy(game)
         copy_game.play_move(best_move)
-        if copy_game.is_round_over():
-            future_reward = copy_game.get_r()
+        immediate_reward = 0
+        future_reward = 0
+        if game.move_ends_game(best_move):
+            immediate_reward = copy_game.get_r()
         else:
-            best_next_move, best_next_move_q = self.decide_best_move(copy_game, player.next())
-            if copy_game.move_ends_game(best_next_move):
-                copy_game.play_move(best_next_move)
-                future_reward = copy_game.get_r()
+            if self.use_montecarlo_random:
+                future_reward = self.monte_carlo_random_search(copy_game, num_explorations=self.random_mc_num_explorations)
             else:
-                future_reward = best_next_move_q
+                future_reward = self.monte_carlo_best_search(copy_game, depth=self.mc_best_move_depth)
 
-        self.record_history_matrices.append(previous_history_matrix)
+        self.record_history_matrices.append(history_matrix)
         self.record_move_vectors.append(move_vector)
         self.record_hand_vectors.append(hand_vector)
 
         # take old value and move by the newly learned value
-        #q_update = (1 - self.learning_rate) * best_move_q + self.learning_rate * future_reward
-        if np.abs(best_move_q - future_reward) > 1 and  len(game.get_game_logs()) < 7:
-            assert False
-            print('Diff')
-            composite = keras.models.load_model('combined15.h5')
-            composite.predict(x=[np.array([self.derive_features(copy_game)]), np.array([move_vector]), np.array([hand_vector])])
-            self.decide_best_move(copy_game, player.next())
-        q_update = best_move_q + self.learning_rate * (self.discount_factor * future_reward - best_move_q)
+        q_update = best_move_q + self.learning_rate * (immediate_reward + self.discount_factor * future_reward - best_move_q)
+        #if abs(int(q_update) - q_update) < 1E-5:
+        #    print('Diff')
 
+        self._record_state_q.append(best_move_q)
         self._record_future_q.append(q_update)
 
     def get_record_history_matrices(self):
@@ -340,24 +378,10 @@ class LearningPlayer_v1(Player):
         return self.__str__()
 
 class RandomPlayer(Player):
-    def make_bet(self, game, player: TurnPosition):
-        if game.get_bet_amount() < game.MAX_BET:
-            return BetMove(game.get_bet_amount() + 1)
-        return None
-
-    def make_move(self, game, player: TurnPosition):
-        hand = CardSet(Counter(game.get_hand(player)))
-        all_moves = hand.get_all_moves()
-        if game.get_control_position() != player:
-            legal_moves = [move for move in all_moves if move.beats(game.get_last_played())]
-        else:
-            # we have control
-            legal_moves = all_moves
-
-        if len(legal_moves) == 0:
-            return None
+    def make_move(self, game, debug=False):
+        legal_moves = game.get_legal_moves()
         return random.choice(legal_moves)
 
 class NoBetPlayer(Player):
-    def make_bet(self, game, player: TurnPosition):
+    def make_move(self, game, debug=False):
         return None
\ No newline at end of file
